module SocialAvatarProxy
  class Configuration
    class Cache
      def read(options = {})
        false
      end
      
      def write(file, options = {})
        file
      end
      
      def fetch(options = {}, &block)
        # if the cache is enabled attempt read
        file = attempt_read(options)
        # if read fails yield the block
        # if the cache is enabled attempt write
        # return the block result
        unless file
          file = block.yield
          attempt_write(file, options) if file
        end
        # return the file
        file
      end
      
      def configure(&block)
        enable
        instance_eval(&block)
      end
      
      def disabled?
        !enabled?
      end

      def enabled?
        !!@enabled
      end
      
      private
      def disable
        @enabled = false
      end
      
      def enable
        @enabled = true
      end
      
      def attempt_read(options = {})
        enabled? && read(options)
      end
      
      def attempt_write(file, options = {})
        enabled? && write(file, options)
      end
    end
  end
end