module SocialAvatarProxy
  class Configuration
    class HttpCache
      def configure(&block)
        @enabled = true
        instance_eval(&block)
      end

      def apply_caching_headers(response)
        # if we want to expire in a set time, calculate the header
        if expires
          response["Expires"] = (Time.now + expires.to_i).httpdate
        end
        # if we want to set cache control settings
        if cc = cache_control
          directives = []
          directives << "no-cache" if cc[:no_cache]
          directives << "max-stale=#{cc[:max_stale]}" if cc[:max_stale]
          directives << "max-age=#{cc[:max_age]}" if cc[:max_age]
          directives << (cc[:public] ? "public" : "private")
          response["Cache-Control"] = directives.join(", ")
        end
        # return the response
        response
      end

      def disabled?
        !enabled?
      end

      def enabled?
        !!@enabled
      end

      private
      def cache_control(value = nil)
        if value
          @cache_control = value
        end
        @cache_control
      end

      def expires(value = nil)
        if value
          @expires = value
        end
        @expires
      end
      
      def disable
        @enabled = false
      end
    end
  end
end